/*
   This program is provided under the LGPL license ver 2.
   Written by Katsumi.
   http://hp.vector.co.jp/authors/VA016157/
   kmorimatsu@users.sourceforge.jp
*/

#include "main.h"
#include "diskio.h"
#include "pff.h"

#define FILENAME_FLASH_ADDRESS (PROGRAM_FLASH_END_ADRESS-15)

static char* g_error_str=0;

void sd_loadMZF(){
	BYTE res;
	static FATFS warea;
	static WORD len;
	int start,size,i;
	char* appname;
	char fname[15];
	// Determine application names
	appname=(char*)FILENAME_FLASH_ADDRESS;
	if (appname[0]<0x30 || 0x7A<appname[0]) {
		appname="BUGFIRE";
	}
	for (i=0;i<8;i++) {
		if (!appname[i]) break;
		fname[i]=appname[i];
	}
	fname[i]='.';
	// Connect to SD card
	res=disk_initialize();
	if (res) {
		g_error_str="DISK INITIALIZATION ERROR";
		return;
	}
	// Connect to FAT file system
	res=pf_mount(&warea);
	if (res) {
		g_error_str="DISK MOUNT ERROR";
		return;
	}
	// Create TXT file name
	for (i=0;i<9;i++) {
		if (fname[i]=='.') break;
	}
	fname[i+1]='T';
	fname[i+2]='X';
	fname[i+3]='T';
	fname[i+4]=0;
	// Load keymap script
	res=pf_open(fname);
	if (res) {
		g_error_str="FILE OPEN ERROR: TXT";
		return;
	}
	// Load script (512 bytes max).
	res=pf_read(&RAM[0x0000],512,&len);
	if (res) {
		g_error_str="FILE READ ERROR: TXT";
		return;
	}
	// Insert 0x00 at the end.
	RAM[len]=0x00;
	// Set keys settings
	keys_set(RAM);
	// Create MZF file name
	for (i=0;i<9;i++) {
		if (fname[i]=='.') break;
	}
	fname[i+1]='M';
	fname[i+2]='Z';
	fname[i+3]='F';
	fname[i+4]=0;
	// Load MZF
	res=pf_open(fname);
	if (res) {
		g_error_str="FILE OPEN ERROR: MZF";
		return;
	}
	// Load header.
	res=pf_read(&RAM[0x00F0],128,&len);
	if (res || len<128) {
		g_error_str="FILE READ ERROR: MZF";
		return;
	}
	size=RAM[0x0103];
	size<<=8;
	size|=RAM[0x0102];
	start=RAM[0x0105];
	start<<=8;
	start|=RAM[0x0104];
	// Check format
	if (start<0x1200 || 0x7FFF<start || 0x7FFF<start+size) {
		g_error_str="FILE FORMAT ERROR";
		return;
	}
	// Load body
	while (128<=size) {
		res=pf_read(&RAM[start-0x1000],128,&len);
		if (res || len<128) {
			g_error_str="FILE LOAD ERROR";
			return;
		}
		size-=128;
		start+=128;
	}
	if (0<size) {
		res=pf_read(&RAM[start-0x1000],size,&len);
		if (res || len<size) {
			g_error_str="FILE LOAD ERROR";
			return;
		}
	}
}

void sd_error(){
	int i;
	if (!g_error_str) return;
	// Clear screen
	for (i=0;i<1000;i++) {
		VRAM[i]=0;
	}
	// Copy string to VRAM
	for (i=0;g_error_str[i];i++) {
		VRAM[40*12+i]=ascii2char[g_error_str[i]];
	}
	// Infinite loop.
	while(1){
		asm volatile("nop");
	}
}